local MAP = 2
local CENTER_X, CENTER_Y, CENTER_Z = -2240*FU, -9216*FU, -128*FU


freeslot("SPR_LJ72", "MT_LJ72_MONOCHROMESPHERE", "S_LJ72_MONOCHROMESPHERE")

mobjinfo[MT_LJ72_MONOCHROMESPHERE] = {
	spawnhealth = 1,
	spawnstate = S_LJ72_MONOCHROMESPHERE,
	radius = 8*FU,
	height = 16*FU,
	flags = MF_NOGRAVITY|MF_NOBLOCKMAP|MF_NOCLIP|MF_NOCLIPHEIGHT
}

states[S_LJ72_MONOCHROMESPHERE] = { SPR_LJ72, A|FF_FULLBRIGHT, -1 }


local function sinCycle(time, low, high, speed)
	local base = sin(time % speed * FU / speed * FU)
	return low + FixedMul(base + FU, (high - low) / 2)
end

local function pointToDist3D(x1, y1, z1, x2, y2, z2)
	return R_PointToDist2(0, z1, R_PointToDist2(x1, y1, x2, y2), z2)
end

local function pointToAngle3D(x1, y1, z1, x2, y2, z2)
	local angle = R_PointToAngle2(x1, y1, x2, y2)
	local vangle = R_PointToAngle2(0, z1, R_PointToDist2(x1, y1, x2, y2), z2)
	return angle, vangle
end

local function randomPointInSphere(x, y, z, radius)
	radius = $ / FU
	local rx, ry, rz
	repeat
		rx = P_RandomRange(-radius, radius) * FU
		ry = P_RandomRange(-radius, radius) * FU
		rz = P_RandomRange(-radius, radius) * FU
	until pointToDist3D(0, 0, 0, rx, ry, rz) <= radius * FU
	return x + rx, y + ry, z + rz
end

local function thrustTowards(mo, x, y, z, speed)
	local angle, vangle = pointToAngle3D(mo.x, mo.y, mo.z, x, y, z)
	mo.momx = $ + FixedMul(FixedMul(speed, cos(angle)), cos(vangle))
	mo.momy = $ + FixedMul(FixedMul(speed, sin(angle)), cos(vangle))
	mo.momz = $ + FixedMul(speed, sin(vangle))
end

local function instaThrustTowards(mo, x, y, z, speed)
	mo.momx, mo.momy, mo.momz = 0, 0, 0
	thrustTowards(mo, x, y, z, speed)
end


addHook("MapLoad", function()
	if gamemap ~= MAP then return end

	local sphere = P_SpawnMobj(CENTER_X, CENTER_Y, CENTER_Z, MT_LJ72_MONOCHROMESPHERE)

	local aura1 = P_SpawnMobj(CENTER_X, CENTER_Y, CENTER_Z, MT_LJ72_MONOCHROMESPHERE)
	sphere.aura1 = aura1
	aura1.frame = A|FF_TRANS70
	aura1.dispoffset = 1

	local aura2 = P_SpawnMobj(CENTER_X, CENTER_Y, CENTER_Z, MT_LJ72_MONOCHROMESPHERE)
	sphere.aura2 = aura2
	aura2.frame = A|FF_TRANS90
	aura2.dispoffset = 2

	local aura3 = P_SpawnMobj(CENTER_X, CENTER_Y, CENTER_Z, MT_LJ72_MONOCHROMESPHERE)
	sphere.aura3 = aura3
	aura3.frame = A|FF_TRANS90
	aura3.dispoffset = 3
end)

addHook("MobjThinker", function(sphere)
	if not sphere.aura1 then return end

	local aura1 = sphere.aura1
	local aura2 = sphere.aura2
	local aura3 = sphere.aura3

	local zOffset = sinCycle(leveltime, -64*FU, 64*FU, 11*TICRATE)
	local z = CENTER_Z + 384*FU + zOffset
	P_MoveOrigin(sphere, sphere.x, sphere.y, z)
	P_MoveOrigin(aura1, aura1.x, aura1.y, z)
	P_MoveOrigin(aura2, aura2.x, aura2.y, z)
	P_MoveOrigin(aura3, aura3.x, aura3.y, z)

	local scale = sinCycle(leveltime, FU*7/4, 2*FU, 5*TICRATE)
	sphere.spritexscale, sphere.spriteyscale = scale, scale

	local auraScale = scale * 9/8
	aura1.spritexscale, aura1.spriteyscale = auraScale, auraScale

	local aura2Scale = scale * 11/8
	aura2.spritexscale, aura2.spriteyscale = aura2Scale, aura2Scale

	local aura3Scale = scale * 14/8
	aura3.spritexscale, aura3.spriteyscale = aura3Scale, aura3Scale

	for _ = 1, 2 do
		local x, y, z = randomPointInSphere(0, 0, 0, 96*scale)
		local p = P_SpawnMobjFromMobj(sphere, x, y, z, MT_LJ72_MONOCHROMESPHERE)
		instaThrustTowards(p, sphere.x, sphere.y, sphere.z, -scale/8)
		p.fuse = TICRATE*3/2
		p.spritexscale, p.spriteyscale = FU/2, FU/2
	end
end, MT_LJ72_MONOCHROMESPHERE)
